@@ -1,35 +1,21 @@
 /*
- *  Copyright (C) 2015 Boudhayan Gupta <bgupta@kde.org>
+ *  SPDX-FileCopyrightText: 2015 Boudhayan Gupta <bgupta@kde.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "ExportMenu.h"
-
-#include "Config.h"
+#include "spectacle_gui_debug.h"
 
 #include <KLocalizedString>
-#include <KMimeTypeTrader>
-#include <KRun>
+#include <KApplicationTrader>
+#include <KIO/ApplicationLauncherJob>
+#include <KNotificationJobUiDelegate>
 #include <KStandardShortcut>
 #ifdef KIPI_FOUND
 #include <KIPI/Plugin>
 #endif
 
-#include <QDebug>
 #include <QJsonArray>
 #include <QTimer>
 
@@ -78,15 +64,23 @@ void ExportMenu::getKServiceItems()
     // populate all locally installed applications and services
     // which can handle images first
 
-    const KService::List services = KMimeTypeTrader::self()->query(QStringLiteral("image/png"));
+    const KService::List services = KApplicationTrader::queryByMimeType(QStringLiteral("image/png"));
 
-    Q_FOREACH (auto service, services) {
+    for (auto service : services) {
         QString name = service->name().replace(QLatin1Char('&'), QLatin1String("&&"));
-        QAction *action = new QAction(QIcon::fromTheme(service->icon()), name, nullptr);
+        QAction *action = new QAction(QIcon::fromTheme(service->icon()), name, this);
 
-        connect(action, &QAction::triggered, [=]() {
-            QList<QUrl> whereIs({ mExportManager->tempSave() });
-            KRun::runService(*service, whereIs, parentWidget(), true);
+        connect(action, &QAction::triggered, this, [=]() {
+            const QUrl filename = mExportManager->getAutosaveFilename();
+            mExportManager->doSave(filename);
+
+            auto *job = new KIO::ApplicationLauncherJob(service);
+            auto *delegate = new KNotificationJobUiDelegate;
+            delegate->setAutoErrorHandlingEnabled(true);
+            job->setUiDelegate(delegate);
+
+            job->setUrls({filename});
+            job->start();
         });
         addAction(action);
     }
@@ -97,13 +91,16 @@ void ExportMenu::getKServiceItems()
     addSeparator();
 
     QAction *openWith = new QAction(this);
-    openWith->setText(i18n("Other Application"));
-    openWith->setIcon(QIcon::fromTheme(QStringLiteral("document-share")));
+    openWith->setText(i18n("Other Application..."));
     openWith->setShortcuts(KStandardShortcut::open());
 
-    connect(openWith, &QAction::triggered, [=]() {
-        QList<QUrl> whereIs({ mExportManager->tempSave() });
-        KRun::displayOpenWithDialog(whereIs, parentWidget(), true);
+    connect(openWith, &QAction::triggered, this, [=]() {
+        const QUrl filename = mExportManager->getAutosaveFilename();
+        mExportManager->doSave(filename);
+        auto job = new KIO::ApplicationLauncherJob;
+        job->setUiDelegate(new KNotificationJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled));
+        job->setUrls({filename});
+        job->start();
     });
     addAction(openWith);
 }
@@ -122,39 +119,51 @@ void ExportMenu::getKipiItems()
     mKipiMenu->clear();
 
     mKipiInterface = new KSGKipiInterface(this);
-    KIPI::PluginLoader *loader = new KIPI::PluginLoader;
+    
+    KIPI::PluginLoader *loader = KIPI::PluginLoader::instance();
+    if (!loader)
+    {
+        // The loader needs to live at least as long as the plugins
+        // loaded through it, since the plugins use the loader's
+        // interface() call to get the KIPI interface they conform to.
+        //
+        // ASAN may complain about a leak here, because this loader
+        // pointer goes out of scope, but it is kept around in
+        // the PluginLoader's static instance()
+        loader = new KIPI::PluginLoader;
+    }
 
     loader->setInterface(mKipiInterface);
     loader->init();
 
     KIPI::PluginLoader::PluginList pluginList = loader->pluginList();
 
-    Q_FOREACH (const auto &pluginInfo, pluginList) {
+    for (const auto &pluginInfo : qAsConst(pluginList)) {
         if (!(pluginInfo->shouldLoad())) {
             continue;
         }
 
         KIPI::Plugin *plugin = pluginInfo->plugin();
         if (!(plugin)) {
-            qWarning() << i18n("KIPI plugin from library %1 failed to load", pluginInfo->library());
+            qCWarning(SPECTACLE_GUI_LOG) << i18n("KIPI plugin from library %1 failed to load", pluginInfo->library());
             continue;
         }
 
         plugin->setup(&mDummyWidget);
 
-        QList<QAction *> actions = plugin->actions();
+        const QList<QAction *> actions = plugin->actions();
         QSet<QAction *> exportActions;
 
-        Q_FOREACH (auto action, actions) {
+        for (auto action : actions) {
             KIPI::Category category = plugin->category(action);
             if (category == KIPI::ExportPlugin) {
                 exportActions += action;
-            } else if (category == KIPI::ImagesPlugin && pluginInfo->library().contains(QStringLiteral("kipiplugin_sendimages"))) {
+            } else if (category == KIPI::ImagesPlugin && pluginInfo->library().contains(QLatin1String("kipiplugin_sendimages"))) {
                 exportActions += action;
             }
         }
 
-        Q_FOREACH (auto action, exportActions) {
+        for (auto action : qAsConst(exportActions)) {
             mKipiMenu->addAction(action);
         }
     }
@@ -172,6 +181,7 @@ void ExportMenu::loadPurposeMenu()
     // attach the menu
     QAction *purposeMenu = addMenu(mPurposeMenu);
     purposeMenu->setText(i18n("Share"));
+    purposeMenu->setIcon(QIcon::fromTheme(QStringLiteral("document-share")));
 
     // set up the callback signal
     connect(mPurposeMenu, &Purpose::Menu::finished, this, [this](const QJsonObject &output, int error, const QString &message) {
@@ -193,7 +203,7 @@ void ExportMenu::loadPurposeItems()
     }
 
     // updated image available, we lazily load it now
-    QString dataUri = ExportManager::instance()->pixmapDataUri();
+    QString dataUri = ExportManager::instance()->tempSave().toString();
     mUpdatedImageAvailable = false;
 
     mPurposeMenu->model()->setInputData(QJsonObject {
