@@ -1,22 +1,6 @@
 /* This file is part of Spectacle, the KDE screenshot utility
- * Copyright 2019 David Redondo <kde@david-redondo.de>
- * Copyright (C) 2015 Boudhayan Gupta <bgupta@kde.org>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- *
+ * SPDX-FileCopyrightText: 2019 David Redondo <kde@david-redondo.de>
+ * SPDX-FileCopyrightText: 2015 Boudhayan Gupta <bgupta@kde.org>
  * SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
@@ -50,7 +34,8 @@
 #include <KGuiItem>
 #include <KHelpMenu>
 #include <KIO/OpenFileManagerWindowJob>
-#include <KRun>
+#include <KIO/OpenUrlJob>
+#include <KIO/JobUiDelegate>
 #include <KLocalizedString>
 #include <KConfigGroup>
 #include <KSharedConfig>
@@ -70,6 +55,9 @@ KSMainWindow::KSMainWindow(Platform::GrabModes theGrabModes, Platform::ShutterMo
     mToolsButton(new QPushButton(this)),
     mSendToButton(new QPushButton(this)),
     mClipboardButton(new QToolButton(this)),
+    mClipboardMenu(new QMenu(this)),
+    mClipboardLocationAction(new QAction(this)),
+    mClipboardImageAction(new QAction(this)),
     mSaveButton(new QToolButton(this)),
     mSaveMenu(new QMenu(this)),
     mSaveAsAction(new QAction(this)),
@@ -137,6 +125,7 @@ void KSMainWindow::init()
 
     connect(ExportManager::instance(), &ExportManager::imageSaved, this, &KSMainWindow::imageSaved);
     connect(ExportManager::instance(), &ExportManager::imageCopied, this, &KSMainWindow::imageCopied);
+    connect(ExportManager::instance(), &ExportManager::imageLocationCopied, this, &KSMainWindow::imageSavedAndLocationCopied);
     connect(ExportManager::instance(), &ExportManager::imageSavedAndCopied, this, &KSMainWindow::imageSavedAndCopied);
 
     // the KSGWidget
@@ -169,12 +158,13 @@ void KSMainWindow::init()
             mKSWidget->showAnnotator();
             mAnnotateButton->setText(i18n("Annotation done"));
         }
-        mAnnotatorActive = !mAnnotatorActive;
 
-        mToolsButton->setEnabled(!mAnnotatorActive);
-        mSendToButton->setEnabled(!mAnnotatorActive);
-        mClipboardButton->setEnabled(!mAnnotatorActive);
-        mSaveButton->setEnabled(!mAnnotatorActive);
+        mToolsButton->setEnabled(mAnnotatorActive);
+        mSendToButton->setEnabled(mAnnotatorActive);
+        mClipboardButton->setEnabled(mAnnotatorActive);
+        mSaveButton->setEnabled(mAnnotatorActive);
+
+        mAnnotatorActive = !mAnnotatorActive;
     });
 
     mDialogButtonBox->addButton(mAnnotateButton, QDialogButtonBox::ActionRole);
@@ -192,9 +182,8 @@ void KSMainWindow::init()
     mSendToButton->setAutoDefault(false);
     mDialogButtonBox->addButton(mSendToButton, QDialogButtonBox::ActionRole);
 
-    mClipboardButton->setDefaultAction(KStandardAction::copy(this, &KSMainWindow::copy, this));
-    mClipboardButton->setText(i18n("Copy to Clipboard"));
-    mClipboardButton->setToolTip(i18n("Copy the current screenshot image to the clipboard."));
+    mClipboardButton->setMenu(mClipboardMenu);
+    mClipboardButton->setPopupMode(QToolButton::MenuButtonPopup);
     mClipboardButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     mDialogButtonBox->addButton(mClipboardButton, QDialogButtonBox::ActionRole);
 
@@ -230,6 +219,18 @@ void KSMainWindow::init()
     mSaveMenu->addAction(mSaveAction);
     setDefaultSaveAction();
 
+    // the clipboard menu
+    mClipboardImageAction = KStandardAction::copy(this, &KSMainWindow::copyImage, this);
+    mClipboardImageAction->setText(i18n("Copy Image to Clipboard"));
+    mClipboardLocationAction = new QAction(
+        QIcon::fromTheme(QStringLiteral("edit-copy")),
+        i18n("Copy Location to Clipboard"), this);
+    connect(mClipboardLocationAction, &QAction::triggered, this, &KSMainWindow::copyLocation);
+    mClipboardMenu->addAction(mClipboardImageAction);
+    mClipboardMenu->addAction(mClipboardLocationAction);
+    setDefaultCopyAction();
+
+
     // message widget
     connect(mMessageWidget, &KMessageWidget::linkActivated, this, [](const QString &str) { QDesktopServices::openUrl(QUrl(str)); } );
 
@@ -254,7 +255,6 @@ void KSMainWindow::init()
     } else if (!mShutterModes.testFlag(Platform::ShutterMode::Immediate)) {
         mKSWidget->lockOnClickEnabled();
     }
-    resize(QSize(DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT).expandedTo(minimumSize()));
 
     // Allow Ctrl+Q to quit the app
     QAction *actionQuit = KStandardAction::quit(qApp, &QApplication::quit, this);
@@ -272,6 +272,11 @@ void KSMainWindow::init()
     // done with the init
 }
 
+QSize KSMainWindow::sizeHint() const
+{
+    return QSize(DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT).expandedTo(minimumSize());
+}
+
 int KSMainWindow::windowWidth(const QPixmap &pixmap) const
 {
     // Calculates what the width of the window should be for the captured image to perfectly fit
@@ -300,6 +305,20 @@ void KSMainWindow::setDefaultSaveAction()
     }
 }
 
+void KSMainWindow::setDefaultCopyAction()
+{
+    switch (Settings::lastUsedCopyMode()) {
+    case Settings::CopyImage:
+        mClipboardButton->setText(i18n("Copy Image to Clipboard"));
+        mClipboardButton->setDefaultAction(mClipboardImageAction);
+        break;
+    case Settings::CopyLocation:
+        mClipboardButton->setText(i18n("Copy Location to Clipboard"));
+        mClipboardButton->setDefaultAction(mClipboardLocationAction);
+        break;
+    }
+}
+
 // overrides
 
 void KSMainWindow::moveEvent(QMoveEvent *event)
@@ -400,7 +419,9 @@ void KSMainWindow::showPrintDialog()
 
 void KSMainWindow::openScreenshotsFolder()
 {
-    new KRun(Settings::defaultSaveLocation(), this);
+    auto job = new KIO::OpenUrlJob(Settings::defaultSaveLocation());
+    job->setUiDelegate(new KIO::JobUiDelegate(KIO::JobUiDelegate::AutoHandlingEnabled, this));
+    job->start();
 }
 
 void KSMainWindow::quit(const QuitBehavior quitBehavior)
@@ -471,8 +492,23 @@ void KSMainWindow::showImageSharedFeedback(bool error, const QString &message)
     }
 }
 
-void KSMainWindow::copy()
+void KSMainWindow::copyLocation()
 {
+    Settings::setLastUsedCopyMode(Settings::CopyLocation);
+    setDefaultCopyAction();
+
+    const bool quitChecked = Settings::quitAfterSaveCopyExport();
+    ExportManager::instance()->doCopyLocationToClipboard();
+    if (quitChecked) {
+        quit(QuitBehavior::QuitExternally);
+    }
+}
+
+void KSMainWindow::copyImage()
+{
+    Settings::setLastUsedCopyMode(Settings::CopyImage);
+    setDefaultCopyAction();
+
     const bool quitChecked = Settings::quitAfterSaveCopyExport();
     ExportManager::instance()->doCopyToClipboard();
     if (quitChecked) {
@@ -486,6 +522,13 @@ void KSMainWindow::imageCopied()
                       KMessageWidget::Information);
 }
 
+void KSMainWindow::imageSavedAndLocationCopied(const QUrl &location)
+{
+    showInlineMessage(i18n("The screenshot has been saved as <a href=\"%1\">%2</a> and its location has been copied to clipboard",
+                           location.toString(), location.fileName()), KMessageWidget::Positive,
+                           MessageDuration::AutoHide, {mOpenContaining});
+}
+
 void KSMainWindow::screenshotFailed()
 {
     showInlineMessage(i18n("Could not take a screenshot. Please report this bug here: <a href=\"https://bugs.kde.org/enter_bug.cgi?product=Spectacle\">create a spectacle bug</a>"),
